##############################################################################
#                               Stripe Plugin
##############################################################################

begin
  require 'stripe'

  RSpec.configure do |config|
    config.after(:each, stripe: true) do
      if defined?(VCR)
        vcr_turned_on = VCR.turned_on?
        VCR.turn_off!
      end

      if defined?(WebMock)
        webmock_net_connect = WebMock.net_connect_allowed?
        WebMock.allow_net_connect!
      end

      begin
        Stripe::Plan.all(count: 100).each do |plan|
          plan.delete if plan.id.match(/test/i) || plan.name.match(/test/i)
        end

        Stripe::Customer.all(count: 100).each(&:delete)
      ensure
        VCR.turn_on!                if defined?(VCR)      && vcr_turned_on
        WebMock.allow_net_connect!  if defined?(WebMock)  && webmock_net_connect
      end
    end
  end
rescue LoadError
end
